; TABSIZE = 8
;  #[ File Description:
;
; JPEG Decoder EQUates and Macros (MC68030)
;
;      1993 Brainstorm & Atari Corp.
;
;  #] File Description: 
;  #[ Structures Macros:

StructInit	macro
X		set	0
		endm

SizeOF		macro	\1
\1		equ	X
		endm

WordEven	macro
		ifne	X-((X/2)*2)
X		set	X+1
		endc
		endm

LongEven	macro
		WordEven
		ifne	X-((X/4)*4)
X		set	X+2
		endc
		endm

Byte		macro
		ifnc	'\1',''
\1		equ	X
		endc
		ifc	'\2',''
X		set	X+1
		elseif
X		set	X+(\2)
		endc
		endm

Struct		macro
\1		equ	X
X		set	X+(\2)
		endm

Word		macro
		ifnc	'\1',''
\1		equ	X
		endc
		ifc	'\2',''
X		set	X+2
		elseif
X		set	X+(2*(\2))
		endc
		endm

Long		macro
		ifnc	'\1',''
\1		equ	X
		endc
		ifc	'\2',''
X		set	X+4
		elseif
X		set	X+(4*(\2))
		endc
		endm

Ptr		macro
		ifnc	'\1',''
\1		equ	X
		endc
		ifc	'\2',''
X		set	X+4
		elseif
X		set	X+(4*(\2))
		endc
		endm

;  #] Structures Macros: 
;  #[ JPEG Decoder Structures:
;	 #[ Cookie Structure:

 StructInit
 Long CookieVersion
 Ptr  CookieOpenDriver
 Ptr  CookieCloseDriver
 Ptr  CookieGetStructSize
 Ptr  CookieGetImageInfo
 Ptr  CookieGetImageSize
 Ptr  CookieDecodeImage
 SizeOF CookieStruct

;	 #] Cookie Structure: 
;	 #[ DSP Structure:

 StructInit
 Long DSPStructMagic			; Always 1
 Long DSPDQTMatrix1,64			; Dequantization Matrix 1
 Long DSPDQTMatrix2,64			; Dequantization Matrix 2
 Long DSPDQTMatrix3,64			; Dequantization Matrix 3
 Long DSPDQTMatrix4,64			; Dequantization Matrix 4
 Long DSPMCUInSize			; Input MCU Size / 2
 Long DSPInComponents			; Input Components Number
 Long Comp1DU				; Number of Data units in Component 1
 Long Comp1DQTMatrix			; Component 1 Dequantization Matrix Address
 Long Comp2DU				; Number of Data units in Component 2
 Long Comp2DQTMatrix			; Component 2 Dequantization Matrix Address
 Long Comp3DU				; Number of Data units in Component 3
 Long Comp3DQTMatrix			; Component 3 Dequantization Matrix Address
 Long Comp4DU				; Number of Data units in Component 4
 Long Comp4DQTMatrix			; Component 4 Dequantization Matrix Address
 Long DSPMCUOutSize			; Output MCU Size / 2
 Long DSPColorSpaceMagic		; Color Space Conversion Magic
 Long DSPSendMCUMagic			; Send MCU Magic (8, 16, 24, 32 Bits)
 Long DSPComp1ExpMagic			; Component 1 Expansion Magic
 Long DSPComp2ExpMagic			; Component 2 Expansion Magic
 Long DSPComp3ExpMagic			; Component 3 Expansion Magic
 Long DSPComp4ExpMagic			; Component 4 Expansion Magic
 SizeOF JPEGDDSPStruct

;	 #] DSP Structure: 
;	 #[ Components Structures:

 StructInit				; Components Huffman Structure
 Word DUPerMCU
 Ptr HuffACTreePointer
 Ptr HuffDCTreePointer
 SizeOF JPEGCompStruct

 StructInit				; Components Expansion Structure
 Word XNumber
 Word YNumber
 SizeOF JPEGExpStruct

;	 #] Components Structures: 
;	 #[ VDI MFDB Structure:

 StructInit
 Ptr  MFDBAddress
 Word MFDBPixelWidth
 Word MFDBPixelHeight
 Word MFDBWordSize
 Word MFDBFormatFlag
 Word MFDBBitPlanes
 Word MFDBReserved,3
 SizeOF MFDBStruct

;	 #] VDI MFDB Structure: 
;	 #[ JPEG Decoder Structure:

 StructInit
;
; Officially Referenced Structure:
;
 Ptr  InPointer				; JPEG Image Pointer
 Ptr  OutPointer			; Output Buffer/Filename Pointer (see OutFlag)
 Long InSize				; JPEG Image Size (Bytes)
 Long OutSize				; Output Image Size (Bytes)
 Word InComponents			; JPEG Image Components Number (1->4)
 Word OutComponents			; Output Components Number (1->4)
 Word OutPixelSize			; Output Pixel Size (1->4)
 Word OutFlag				; 0 (RAM Output) / -1 (Disk Output)
 Word XLoopCounter			; Number of MCUs per Row
 Word YLoopCounter			; Number of MCUs per Column
 Ptr  CreateOutputPtr			; Pointer to User Routine / 0
 Ptr  WriteOutputPtr			; Pointer to User Routine / 0
 Ptr  CloseOutputPtr			; Pointer to User Routine / 0
 Ptr  SigTermOutputPtr			; Pointer to User Routine / 0
 Ptr  Comp1GammaPtr			; Component 1 Gamma Table / 0
 Ptr  Comp2GammaPtr			; Component 2 Gamma Table / 0
 Ptr  Comp3GammaPtr			; Component 3 Gamma Table / 0
 Ptr  Comp4GammaPtr			; Component 4 Gamma Table / 0
 Ptr  UserRoutinePtr			; Pointer to User Routine (Called during Decompression) / 0
 Ptr  OutTmpPointer			; Current OutPointer / Temporary Disk Buffer Pointer (see OutFlag)
 Word MCUsCounter			; Number of MCUs not Decoded
 Word OutTmpHeight			; Number of Lines in OutTmpPointer
 Long UserLong1				; Free, Available for User
 Long UserLong2				; Free, Available for User
 Word OutHandle				; 0 / Output File Handle (see OutFlag)
 Struct OutMFDB,MFDBStruct		; Output Image MFDB
;
; Internal Huffman Variables:
;
 Ptr  HuffDCPointers,2			; Huffman DC Buffers Pointers
 Ptr  HuffACPointers,2			; Huffman AC Buffers Pointers
 Ptr  HuffDCTrees,2			; Huffman DC Trees Pointers
 Ptr  HuffACTrees,2			; Huffman AC Trees Pointers
 Byte HuffDCFlags,2			; Huffman DC Trees Flags
 Byte HuffACFlags,2			; Huffman AC Trees Flags
 Word LastDCValue,4			; Last DC Coefficient Values
 Word HuffmanCurBit			; Current Huffman Bits Number
 Word HuffmanCurStream			; Currenr Huffman Bits
 Long HuffTmpTable,17			; Huffman Temporary Buffer
;
; Internal Dequantization Variables:
;
 Byte QuantTableFlags,4			; Quantization Table Flags
;
; Internal DSP Structure:
;
 Struct DSPStruct,JPEGDDSPStruct
;
; Internal Restart Interval Variables
;
 Byte DRIInternalFlag			; Restart Enabled/Disabled Flag
 Byte RSTNumber				; Expected Restart Marker (0->7)
 Word DRISize				; Restart Interval Size
 Word DRICurSize			; Restart Interval Current Size
;
; Internal Interlacing Variables
;
 Long InterlacingRoutine
 Long InterlacingOffsets,16
 Long OutInternalSize			; Row Size (RAM Output) / 0 (Disk Output)
;
; Internal Miscellaneous Variables & Structures:
;
 Word UserHandle			; User Unique Handle (1->32)
 Struct CompStruct,JPEGCompStruct*4	; Maximum 4 Components
 Word ,1				; + (-1)
 Struct ExpStruct,JPEGExpStruct*4	; 4 Expansion Structures
 Word ,1				; + (-1)
 Word DUsPerMCU				; Number of Data Units per MCU
 Word MaxXSampling			; Maximum X Sampling
 Word MaxYSampling			; Maximum Y Sampling
 Word ExpandOffset			; Current Expand Table Offset
 Word ExpandSize			; XSampling * YSampling (MUST BE <= 10)
 Ptr  MCUInBuffer			; Huffman Decoder's Output Buffer
 Ptr  MCUEndInBuffer
 Ptr  MCUOutBuffer			; DSP Output Buffer
 Long ComponentsInternal,256		; Flag, X Sampling, Y Sampling, Quantization Selector
 Long InCurSize
 Word ComponentsIDs,4
 Word ,1
 Word JPEGMode				; Image Compression type
 Ptr  DSPReceivePointer			; 'Receive DSP Datas' Routine Pointer
 Word DSPReceiveSize			; 'Receive DSP Datas' Datas Size
 Word DNLInternalFlag			; DNL Expected Flag
 SizeOF JPEGDecoderStruct

;	 #] JPEG Decoder Structure: 
;  #] JPEG Decoder Structures: 
;  #[ JPEG Decoder Macros:

GetHuffWord	macro			; Huffman Macro
		move.b	(a5)+,d0
		add.w	0(a4,d0.w*2),a5
		move.b	(a5)+,d2
		add.w	($200.w,a4,d2.w*2),a5
		lsl.w	#8,d0
		move.b	d2,d0
		moveq	#$f,d2
		endm

GetNextBit	macro			; Huffman Macro
		add.w	d0,d0
		subx.w	d1,d1
		move.w	(a2,d1.w*2),d1
		add.w	d1,a2
		endm

;  #] JPEG Decoder Macros: 
;  #[ JPEG Decoder EQUates:
;
; JPEG Structure Release Number
;
VERSION		equ	1

;
; Language EQUates
;
ENGLISH		equ	1
FRENCH		equ	0
GERMAN		equ	0
ITALIAN		equ	0
SPANISH		equ	0

;
; Message returned by the decoder
;
NOERROR		equ	0		; File correctly uncompressed
UNKNOWNFORMAT	equ	1		; File is not JFIF (Error)
INVALIDMARKER	equ	2		; Reserved CCITT Marker Found (Error)
SOF1MARKER	equ	3		; Mode not handled by the decoder (Error)
SOF2MARKER	equ	4		; Mode not handled by the decoder (Error)
SOF3MARKER	equ	5		; Mode not handled by the decoder (Error)
SOF5MARKER	equ	6		; Mode not handled by the decoder (Error)
SOF6MARKER	equ	7		; Mode not handled by the decoder (Error)
SOF7MARKER	equ	8		; Mode not handled by the decoder (Error)
SOF9MARKER	equ	9		; Mode not handled by the decoder (Error)
SOF10MARKER	equ	10		; Mode not handled by the decoder (Error)
SOF11MARKER	equ	11		; Mode not handled by the decoder (Error)
SOF13MARKER	equ	12		; Mode not handled by the decoder (Error)
SOF14MARKER	equ	13		; Mode not handled by the decoder (Error)
SOF15MARKER	equ	14		; Mode not handled by the decoder (Error)
RSTmMARKER	equ	15		; Unexpected RSTm found (Error)
BADDHTMARKER	equ	16		; Buggy DHT Marker (Error)
DACMARKER	equ	17		; Marker not handled by the decoder (Error)
BADDQTMARKER	equ	18		; Buggy DQT Marker (Error)
DNLMARKER	equ	19		; Marker not handled by the decoder (Error)
BADDRIMARKER	equ	20		; Invalid DRI Header Size (Error)
DHPMARKER	equ	21		; Marker not handled by the decoder (Error)
EXPMARKER	equ	22		; Marker not handled by the decoder (Error)
BADSUBSAMPLING	equ	23		; Invalid components subsampling (Error)
NOTENOUGHMEMORY	equ	24		; Not enough memory... (Error)
DECODERBUSY	equ	25		; Decoder is busy (Error)
DSPBUSY		equ	26		; Can't lock the DSP (Error)
BADSOFnMARKER	equ	27		; Buggy SOFn marker (Error)
BADSOSMARKER	equ	28		; Buggy SOS marker (Error)
HUFFMANERROR	equ	29		; Buggy Huffman Stream (Error)
BADPIXELFORMAT	equ	30		; Invalid Output Pixel Format (Error)
DISKFULL	equ	31		; Hard/Floppy Disk Full (Error)
MISSINGMARKER	equ	32		; Marker expected but not found (Error)
IMAGETRUNCATED	equ	33		; More bytes Needed (Error)
EXTRABYTES	equ	34		; Dummy Bytes after EOI Marker (Warning)
USERABORT	equ	35		; User Routine signaled 'Abort'
DSPMEMORYERROR	equ	36		; Not Enough DSP RAM (Error)
NORSTmMARKER	equ	37		; RSTm Marker expected but not found
BADRSTmMARKER	equ	38		; Invalid RSTm Marker Number
DRIVERCLOSED	equ	39		; Driver is Already Closed.
ENDOFIMAGE	equ	40		; Stop Decoding (Internal Message, Should Never Appear)

;                         
; Internal Markers EQUates
;
SOF0		equ	+1
SOF1		equ	+2
SOF2		equ	+3
SOF3		equ	+4
SOF9		equ	+5
SOF10		equ	+6
SOF11		equ	+7

;
; DSP EQUates (ConvertMCU)
;
YTOY		equ	0		; 1 -> 1
YTORGB		equ	1		; 1 -> 3
YCBCRTOY	equ	2		; 3 -> 1
YCBCRTORGB	equ	3		; 3 -> 3

;
; DSP EQUates (SendMCU)
;
SEND8BITSY	equ	0		; Y
SEND16BITSRGB	equ	1		; RGB, TWO COMPONENTS
SEND24BITSRGB	equ	2		; RGB
SEND32BITSRGB	equ	3		; RGB

;
; DSP EQUates (ExpandDU)
;
NOSUBSAMP	equ	-1
X1Y1_X1Y1	equ	0
X1Y2_X1Y1	equ	1
X1Y2_X1Y2	equ	2
X1Y3_X1Y1	equ	3
X1Y3_X1Y3	equ	4
X1Y4_X1Y1	equ	5
X1Y4_X1Y2	equ	6
X1Y4_X1Y4	equ	7
X2Y1_X1Y1	equ	8
X2Y1_X2Y1	equ	9
X2Y2_X1Y1	equ	10
X2Y2_X1Y2	equ	11
X2Y2_X2Y1	equ	12
X2Y2_X2Y2	equ	13
X2Y3_X1Y1	equ	14
X2Y3_X1Y3	equ	15
X2Y3_X2Y1	equ	16
X2Y3_X2Y3	equ	17
X2Y4_X1Y1	equ	18
X2Y4_X1Y2	equ	19
X2Y4_X1Y4	equ	20
X2Y4_X2Y1	equ	21
X2Y4_X2Y2	equ	22
X2Y4_X2Y4	equ	23
X3Y1_X1Y1	equ	24
X3Y1_X3Y1	equ	25
X3Y2_X1Y1	equ	26
X3Y2_X1Y2	equ	27
X3Y2_X3Y1	equ	28
X3Y2_X3Y2	equ	29
X3Y3_X1Y1	equ	30
X3Y3_X1Y3	equ	31
X3Y3_X3Y1	equ	32
X3Y3_X3Y3	equ	33
X3Y4_X3Y4	equ	34
X4Y1_X1Y1	equ	35
X4Y1_X2Y1	equ	36
X4Y1_X4Y1	equ	37
X4Y2_X1Y1	equ	38
X4Y2_X1Y2	equ	39
X4Y2_X2Y1	equ	40
X4Y2_X2Y2	equ	41
X4Y2_X4Y1	equ	42
X4Y2_X4Y2	equ	43
X4Y3_X4Y3	equ	44
X4Y4_X4Y4	equ	45

;
; DSP EQUates (Dequantization Buffers)
;
MATRIX1		equ	$40
MATRIX2		equ	$60
MATRIX3		equ	$80
MATRIX4		equ	$a0

;
; DSP EQUates (Gamma Table Buffers)
;
GAMMAPOINTER	equ	$2000
GAMMASIZE	equ	$200

;  #] JPEG Decoder EQUates: 
